--- drakma-0.4.2-orig/request.lisp	2006-09-07 06:48:04.000000000 -0400
+++ drakma-0.4.2-autoreferer/request.lisp	2006-09-24 18:40:41.000000000 -0400
@@ -153,6 +153,7 @@
                               additional-headers
                               (redirect 5)
                               (redirect-methods '(:get :head))
+                              autoreferer
                               keep-alive
                               (close t)
                               (external-format-out *drakma-default-external-format*)
@@ -287,6 +288,9 @@
 If REDIRECT is true, Drakma will follow redirects \(return codes
 301, 302, 303, or 307) unless REDIRECT is 0.  If REDIRECT is an
 integer, it will be decreased by 1 with each redirect.
+Furthermore, if AUTOREFERER is true, when following redirects,
+Drakma will populate the `Referer' header with the URI that
+triggered the redirection.
 
 If KEEP-ALIVE is T, the server will be asked to keep the
 connection alive, i.e. not to close it after the reply has been
@@ -526,6 +530,8 @@
                                          "Status code was ~A, but ~
 ~:[REDIRECT is ~S~;redirection limit has been exceeded~]."
                                          status-code (integerp redirect) redirect))
+                               (when autoreferer
+                                 (setq additional-headers (set-referer uri additional-headers)))
                                (let* ((location (header-value :location headers))
                                       (new-uri (merge-uris location uri))
                                       ;; can we re-use the stream?
@@ -549,6 +555,7 @@
                                                        (not must-close)
                                                        (not force-ssl)
                                                        http-stream)
+                                          :additional-headers additional-headers
                                           args))))
                              (let ((transfer-encodings (header-value :transfer-encoding headers)))
                                (when transfer-encodings
@@ -600,4 +607,13 @@
         ;; the cleanup form of the UNWIND-PROTECT above
         (when (and http-stream must-close (not want-stream)
                    (not (eq content :continuation)))
-          (ignore-errors (close http-stream)))))))
\ No newline at end of file
+          (ignore-errors (close http-stream)))))))
+
+(defun set-referer (referer-uri &optional alist)
+  "Returns a new copy of the (optional) alist of HTTP headers, with the
+   Referer header set to the given URI."
+  (let ((old-without (remove-if
+                      (lambda (x) (string= "Referer" (car x))) alist)))
+    (if referer-uri
+        (acons "Referer" referer-uri old-without)
+        old-without)))
